@@ -1,4 +1,4 @@
-/* Copyright (C) 2002, 2003 Thorsten Kukuk
+/* Copyright (C) 2002, 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -42,6 +42,7 @@
 #include "public.h"
 #include "logindefs.h"
 #include "read-files.h"
+#include "utf8conv.h"
 
 #ifdef USE_LDAP
 #include "libldap.h"
@@ -357,7 +358,7 @@ main (int argc, char *argv[])
 #endif
 	  else
 	    {
-	      fprintf (stderr, _("Service \"%s\" not supported\n"), optarg);
+	      fprintf (stderr, _("Service `%s' not supported\n"), optarg);
 	      print_usage (stderr, program);
 	      return E_BAD_ARG;
 	    }
@@ -369,7 +370,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -426,7 +427,7 @@ main (int argc, char *argv[])
       /* if we show/modify the data for another user, get the data from
 	 this one.  */
       if (argc == 1)
-	arg_user = argv[0];
+	arg_user = locale_to_utf8 (argv[0]);
       else
 	arg_user = pw->pw_name;
 
@@ -434,11 +435,12 @@ main (int argc, char *argv[])
       if (pw_data == NULL || pw_data->service == S_NONE)
 	{
 	  if (use_service)
-	    fprintf (stderr, _("%s: User `%s' is not known to service `%s'.\n"),
-		     program, arg_user, use_service);
+	    fprintf (stderr,
+		     _("%s: User `%s' is not known to service `%s'.\n"),
+		     program, utf8_to_locale (arg_user), use_service);
 	  else
 	    fprintf (stderr, _("%s: Unknown user `%s'.\n"), program,
-		     arg_user);
+		     utf8_to_locale (arg_user));
 	  return E_UNKNOWN_USER;
 	}
     }
@@ -531,7 +533,8 @@ main (int argc, char *argv[])
     {
       if (uid != 0 && strcmp (caller_name, pw_data->pw.pw_name) != 0)
 	{
-	  fprintf (stderr, _("You can only list your own aging information.\n"));
+	  fprintf (stderr,
+		   _("You can only list your own aging information.\n"));
 	  return E_NOPERM;
 	}
 
@@ -548,7 +551,7 @@ main (int argc, char *argv[])
 	print_shadow_info (pw_data);
       else
 	fprintf (stdout, _("No aging information available for %s.\n"),
-		 pw_data->pw.pw_name);
+		 utf8_to_locale (pw_data->pw.pw_name));
 
       return 0;
     }
@@ -568,7 +571,7 @@ main (int argc, char *argv[])
 
       if (!silent)
 	printf (_("Changing aging information for %s.\n"),
-		pw_data->pw.pw_name);
+		utf8_to_locale (pw_data->pw.pw_name));
 
       if ((res = change_shadow_info (pw_data)) != 0)
 	{
@@ -615,8 +618,8 @@ main (int argc, char *argv[])
 			strtol (lastday, &cp, 10)) == 0 && *cp) ||
 		      pw_data->spn.sp_lstchg < -1)
 		    {
-		      fprintf (stderr, _("Lastday is no date and no\
- integer value >= -1\n"));
+		      fprintf (stderr,
+			_("Lastday is no date and no integer value >= -1\n"));
 		      ++error;
 		    }
 		}
@@ -636,8 +639,7 @@ main (int argc, char *argv[])
 			strtol (expiredate, &cp, 10)) == 0 && *cp) ||
 		      pw_data->spn.sp_expire < -1)
 		    {
-		      fprintf (stderr, _("Expiredate is no date and no\
- integer value >= -1\n"));
+		      fprintf (stderr, _("Expiredate is no date and no integer value >= -1\n"));
 		      ++error;
 		    }
 		}
