@@ -39,8 +39,14 @@
 #endif
 
 #include "public.h"
+#include "utf8conv.h"
 #include "logindefs.h"
 #include "error_codes.h"
+#include "read-files.h"
+
+#ifdef USE_LDAP
+#include "libldap.h"
+#endif
 
 #ifndef _
 #define _(String) gettext (String)
@@ -90,7 +96,7 @@ print_help_shadow (const char *program)
 static void
 print_usage_util (FILE *stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-f name] [-o office] [-p office-phone]\n       [-h home-phone] [-m other] [-q] [-u] [-v] [user]\n"),
+  fprintf (stream, _("Usage: %s [-D binddn] [-P path] [-f name] [-o office] [-p office-phone]\n       [-h home-phone][-m other] [-q] [-u] [-v] [user]\n"),
 	   program);
 }
 
@@ -100,6 +106,12 @@ print_help_util (const char *program)
   print_usage_util (stdout, program);
   fprintf (stdout, _("%s - change user name and information\n\n"),
 	   program);
+#ifdef USE_LDAP
+  fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
+         stdout);
+#endif
+  fputs (_("  -P path        Search passwd and shadow file in \"path\"\n"),
+         stdout);
   fputs (_("  -f full-name   Change your real name\n"), stdout);
   fputs (_("  -o office      Change your office room number\n"), stdout);
   fputs (_("  -p phone       Change your office phone number\n"), stdout);
@@ -217,7 +229,7 @@ check_field (const char *program, const char *field, const char *illegal)
   for (i = 0; i < strlen (field); i++)
     {
       c = field[i];
-      if (strchr(illegal, c) != NULL || c == '"' || c == '\n')
+      if (strchr (illegal, c) != NULL || c == '"' || c == '\n')
 	{
 	  printf (_("%s: '%c' is not allowed.\n"), program, c);
 	  return 1;
@@ -228,6 +240,7 @@ check_field (const char *program, const char *field, const char *illegal)
 	  return 1;
         }
     }
+
   return 0;
 }
 
@@ -239,9 +252,13 @@ main (int argc, char *argv[])
   char *use_service = NULL;
   char *caller_name = NULL;
   char *new_gecos = NULL;
+  char *locale_name;
   int interactive = 1;
   fn_info new;
   int silent = 0;
+#ifdef USE_LDAP
+  char *binddn = NULL;
+#endif
   static struct option long_options_shadow[] = {
     {"full-name",   required_argument, NULL, 'f' },
     {"room",        required_argument, NULL, 'r' },
@@ -257,6 +274,10 @@ main (int argc, char *argv[])
   };
   static char *optstring_shadow = "f:r:w:h:o:qvu";
   static struct option long_options_util[] = {
+#ifdef USE_LDAP
+    {"binddn",      required_argument, NULL, 'D' },
+#endif
+    {"path",        required_argument, NULL, 'P' },
     {"full-name",   required_argument, NULL, 'f' },
     {"office",      required_argument, NULL, 'o' },
     {"phone",       required_argument, NULL, 'p' },
@@ -269,7 +290,7 @@ main (int argc, char *argv[])
     {"help",        no_argument,       NULL, '\255' },
     {NULL,          0,                 NULL, '\0'}
   };
-  static char *optstring_util = "f:o:p:h:r:m:qvu";
+  static char *optstring_util = "D:P:f:o:p:h:r:m:qvu";
   struct option *long_options;
   char *optstring;
   char *envstr;
@@ -283,7 +304,7 @@ main (int argc, char *argv[])
     use_service = "nis";
   else if (strcasecmp (program, "chfn") != 0)
     {
-      fprintf (stderr, _("%s: don't know what I should do.\n"), program);
+      fprintf (stderr, _("%s: Don't know what I should do.\n"), program);
       return 1;
     }
 
@@ -332,7 +353,7 @@ main (int argc, char *argv[])
 	      fprintf (stderr, _("%s: Permission denied.\n"), program);
 	      return 1;
 	    }
-	  new.fullname = strdup (optarg);
+	  new.fullname = locale_to_utf8 (optarg);
 	  interactive = 0;
 	  break;
 	case 'r':
@@ -351,7 +372,7 @@ main (int argc, char *argv[])
 		  fprintf (stderr, _("%s: Permission denied.\n"), program);
 		  return 1;
 		}
-	      new.other = strdup (optarg);
+	      new.other = locale_to_utf8 (optarg);
 	      interactive = 0;
 	    }
 	  else if ((shadow_chfn && c == 'r') ||
@@ -363,7 +384,7 @@ main (int argc, char *argv[])
 		  fprintf (stderr, _("%s: Permission denied.\n"), program);
 		  return 1;
 		}
-	      new.roomno = strdup (optarg);
+	      new.roomno = locale_to_utf8 (optarg);
 	      interactive = 0;
 	    }
 	  else
@@ -386,12 +407,14 @@ main (int argc, char *argv[])
 		use_service = "nisplus";
 	      else if (strcasecmp (optarg, "files") == 0)
 		use_service = "files";
+#ifdef USE_LDAP
 	      else if (strcasecmp (optarg, "ldap") == 0)
 		use_service = "ldap";
+#endif
 	      else
 		{
 		  fprintf (stderr,
-			   _("Service \"%s\" not supported\n"), optarg);
+			   _("Service `%s' not supported.\n"), optarg);
 		  if (shadow_chfn)
 		    print_usage_shadow (stderr, program);
 		  else
@@ -407,7 +430,7 @@ main (int argc, char *argv[])
 	      fprintf (stderr, _("%s: Permission denied.\n"), program);
 	      return 1;
 	    }
-	  new.work_phone = strdup (optarg);
+	  new.work_phone = locale_to_utf8 (optarg);
 	  interactive = 0;
 	  break;
 	case 'h':
@@ -416,12 +439,27 @@ main (int argc, char *argv[])
 	      fprintf (stderr, _("%s: Permission denied.\n"), program);
 	      return 1;
 	    }
-	  new.home_phone = strdup (optarg);
+	  new.home_phone = locale_to_utf8 (optarg);
 	  interactive = 0;
 	  break;
 	case 'q':
 	  silent = 1;
 	  break;
+#ifdef USE_LDAP
+        case 'D':
+          binddn = optarg;
+          break;
+#endif
+        case 'P':
+          if (uid != 0)
+            {
+              fprintf (stderr,
+                       _("Only root is allowed to specify another path\n"));
+              return E_NOPERM;
+            }
+          else
+            files_etc_dir = strdup (optarg);
+          break;
         case '\255':
 	  if (shadow_chfn)
 	    print_help_shadow (program);
@@ -429,7 +467,7 @@ main (int argc, char *argv[])
 	    print_help_util (program);
           return 0;
         case 'v':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case 'u':
 	  if (shadow_chfn)
@@ -448,7 +486,7 @@ main (int argc, char *argv[])
 
   if (argc > 1)
     {
-      fprintf (stderr, _("%s: Too many arguments\n"), program);
+      fprintf (stderr, _("%s: Too many arguments.\n"), program);
       print_error (program);
       return 1;
     }
@@ -480,7 +518,7 @@ main (int argc, char *argv[])
 	 data, too.  */
       if (argc == 1)
 	{
-	  char *user = argv[0];
+	  char *user = locale_to_utf8 (argv[0]);
 
 	  while (getpwnam_r (user, &resultbuf, buffer, buflen, &pw) != 0
 		 && errno == ERANGE)
@@ -489,9 +527,12 @@ main (int argc, char *argv[])
 	      buflen += 256;
 	      buffer = alloca (buflen);
 	    }
+	  free (user);
+
 	  if (!pw)
 	    {
-	      fprintf (stderr, _("%s: Unknown user `%s'.\n"), program, user);
+	      fprintf (stderr, _("%s: Unknown user `%s'.\n"),
+		       program, argv[0]);
 	      return 1;
 	    }
 	}
@@ -500,14 +541,18 @@ main (int argc, char *argv[])
       if (pw_data == NULL || pw_data->service == S_NONE)
 	{
 	  if (use_service)
-	    fprintf (stderr, _("%s: User `%s' is not known to service `%s'.\n"),
-		     program, pw->pw_name, use_service);
+	    fprintf (stderr,
+		     _("%s: User `%s' is not known to service `%s'.\n"),
+		     program, utf8_to_locale (pw->pw_name), use_service);
 	  else
-	    fprintf (stderr, _("%s: Unknown user %s\n"), program,
-		     pw->pw_name);
+	    fprintf (stderr, _("%s: Unknown user `%s'.\n"), program,
+		     utf8_to_locale (pw->pw_name));
 	  return 1;
 	}
-    }
+
+      locale_name = utf8_to_locale (pw_data->pw.pw_name);
+  }
+
 #ifdef WITH_SELINUX
   if (is_selinux_enabled () > 0)
     {
@@ -519,8 +564,8 @@ main (int argc, char *argv[])
             user_context =
               (security_context_t) strdup (_("Unknown user context"));
 	  fprintf (stderr,
-		   _("%s: %s is not authorized to change the finger information for %s\n"),
-		   program, user_context, pw_data->pw.pw_name);
+		   _("%s: %s is not authorized to change the finger information for %s.\n"),
+		   program, user_context, locale_name);
 	  if (security_getenforce() > 0)
 	    {
 	      syslog (LOG_ALERT,
@@ -532,7 +577,7 @@ main (int argc, char *argv[])
 	  else
 	    {
 	      fprintf (stderr,
-		       _("SELinux is in permissive mode, continuing\n"));
+		       _("SELinux is in permissive mode, continuing.\n"));
 	      freecon (user_context);
 	    }
 	}
@@ -540,23 +585,56 @@ main (int argc, char *argv[])
 #endif
 
   /* Only root is allowed to change the gecos field for local users. */
-  if (uid && uid != pw_data->pw.pw_uid && pw_data->service == S_LOCAL)
+  if (uid && uid != pw_data->pw.pw_uid &&
+      (pw_data->service == S_LOCAL
+#ifdef USE_LDAP
+       || (pw_data->service == S_LDAP && binddn == NULL)
+#endif
+       ))
     {
-      syslog (LOG_ERR, "%u cannot change finger information for \"%s\"", uid,
-	      pw_data->pw.pw_name);
-      fprintf(stderr, _("You cannot change the finger information for %s.\n"),
-	      pw_data->pw.pw_name);
+      syslog (LOG_ERR, "%u cannot change finger information for `%s'",
+	      uid, pw_data->pw.pw_name);
+      fprintf (stderr,
+	       _("You cannot change the finger information for `%s'.\n"),
+	       locale_name);
       free_user_t (pw_data);
       return 1;
     }
 
   if (!silent)
-    printf (_("Changing finger information for %s.\n"), pw_data->pw.pw_name);
+    printf (_("Changing finger information for %s.\n"), locale_name);
 
   if (getlogindefs_bool ("CHFN_AUTH", 1) || pw_data->service != S_LOCAL)
     {
-      if (do_authentication (program, caller_name, pw_data) != 0)
-	return 1;
+#ifdef USE_LDAP
+      if (binddn && pw_data->service == S_LDAP)
+	{
+	  /* A user tries to change data stored in a LDAP database and
+	     knows the Manager dn, now we need the password from him.  */
+	  ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
+	  char *cp;
+
+	  if (session == NULL)
+	    return E_FAILURE;
+
+	  cp = getpass (_("Enter LDAP Password:"));
+
+	  pw_data->binddn = strdup (binddn);
+
+	  if (open_ldap_session (session) != 0)
+	    return E_FAILURE;
+
+	  if (ldap_authentication (session, NULL, binddn, cp) != 0)
+	    return E_NOPERM;
+
+	  close_ldap_session (session);
+
+	  pw_data->oldclearpwd = strdup (cp);
+	}
+      else
+#endif /* USE_LDAP */
+	if (do_authentication (program, caller_name, pw_data) != 0)
+	  return 1;
       if (pw_data->service != S_LOCAL && get_old_clear_password (pw_data) != 0)
 	return 1;
     }
@@ -575,40 +653,40 @@ main (int argc, char *argv[])
       new.fullname = strdup (old.fullname ?:"");
     if (check_field (program, new.fullname, ":,=") != 0)
       {
-	fprintf (stderr, _("%s: invalid name: \"%s\"\n"), program,
-		 new.fullname);
+	fprintf (stderr, _("%s: Invalid name: `%s'\n"), program,
+		 utf8_to_locale (new.fullname));
 	return 1;
       }
     if (!new.roomno)
       new.roomno = strdup (old.roomno ?:"");
     if (check_field (program, new.roomno, ":,=") != 0)
       {
-	fprintf (stderr, _("%s: invalid room number: \"%s\"\n"), program,
-		 new.roomno);
+	fprintf (stderr, _("%s: Invalid room number: `%s'\n"), program,
+		 utf8_to_locale (new.roomno));
 	return 1;
       }
     if (!new.work_phone)
       new.work_phone = strdup (old.work_phone ?:"");
     if (check_field (program, new.work_phone, ":,=") != 0)
       {
-	fprintf (stderr, _("%s: invalid work phone: \"%s\"\n"), program,
-		 new.work_phone);
+	fprintf (stderr, _("%s: Invalid work phone: `%s'\n"), program,
+		 utf8_to_locale (new.work_phone));
 	return 1;
       }
     if (!new.home_phone)
       new.home_phone = strdup (old.home_phone ?:"");
     if (check_field (program, new.work_phone, ":,=") != 0)
       {
-	fprintf (stderr, _("%s: invalid home phone: \"%s\"\n"), program,
-		 new.home_phone);
+	fprintf (stderr, _("%s: Invalid home phone: `%s'\n"), program,
+		 utf8_to_locale (new.home_phone));
 	return 1;
       }
     if (!new.other)
       new.other = strdup (old.other ?:"");
     if (check_field (program, new.other, ":") != 0)
       {
-	fprintf (stderr, _("%s: \"%s\" contains illegal characters\n"),
-		 program, new.fullname);
+	fprintf (stderr, _("%s: `%s' contains illegal characters.\n"),
+		 program, utf8_to_locale (new.other));
 	return 1;
       }
 
