@@ -39,9 +39,15 @@
 #endif
 
 #include "public.h"
+#include "utf8conv.h"
 #include "logindefs.h"
+#include "read-files.h"
 #include "error_codes.h"
 
+#ifdef USE_LDAP
+#include "libldap.h"
+#endif
+
 #ifndef _
 #define _(String) gettext (String)
 #endif
@@ -49,7 +55,7 @@
 static void
 print_usage (FILE *stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-s shell] [-l] [-q] [--help] [--usage] [--version] [user]\n"),
+  fprintf (stream, _("Usage: %s [-D binddn] [-P path] [-s shell] [-l] [-q]\n            [--help] [--usage] [--version] [user]\n"),
 	   program);
 }
 
@@ -58,6 +64,13 @@ print_help (const char *program)
 {
   print_usage (stdout, program);
   fprintf (stdout, _("%s - change login shell\n\n"), program);
+
+#ifdef USE_LDAP
+  fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
+         stdout);
+#endif
+  fputs (_("  -P path        Search passwd and shadow file in \"path\"\n"),
+         stdout);
   fputs (_("  -l             List allowed shells from /etc/shells\n"),
          stdout);
   fputs (_("  -s shell       Use 'shell' as new login shell\n"), stdout);
@@ -69,7 +82,8 @@ print_help (const char *program)
   fputs (_("  -v, --version  Print program version\n"), stdout);
 
   if (strcmp (program, "chsh") == 0)
-    fputs (_("Valid services for -r are: files, nis, nisplus\n"), stdout);
+    fputs (_("Valid services for -r are: files, ldap, nis, nisplus\n"),
+	   stdout);
 }
 
 /* If the given shell appears in /etc/shells, return 1. If not,
@@ -164,19 +178,19 @@ check_shell (const char *program, const char *shell)
 
   if (*shell != '/')
     {
-      fprintf (stderr, _("%s: shell must be a full path name.\n"), program);
+      fprintf (stderr, _("%s: Shell must be a full path name.\n"), program);
       if (uid)
 	return 1;
     }
   if (access (shell, F_OK) < 0)
     {
-      fprintf (stderr, _("%s: \"%s\" does not exist.\n"), program, shell);
+      fprintf (stderr, _("%s: `%s' does not exist.\n"), program, shell);
       if (uid)
 	return 1;
     }
   if (access (shell, X_OK) < 0)
     {
-      fprintf (stderr, _("%s: \"%s\" is not executable.\n"), program, shell);
+      fprintf (stderr, _("%s: `%s' is not executable.\n"), program, shell);
       if (uid)
 	return 1;
     }
@@ -204,7 +218,7 @@ check_shell (const char *program, const char *shell)
 	{
 	  fprintf (stderr, _("%s: \"%s\" is not listed in /etc/shells.\n"),
 		   program, shell);
-	  fprintf (stderr, _("%s: use -l option to see list.\n"), program);
+	  fprintf (stderr, _("%s: Use -l option to see list.\n"), program);
 	  return 1;
 	}
     }
@@ -222,7 +236,11 @@ main (int argc, char *argv[])
   user_t *pw_data = NULL;
   char *use_service = NULL;
   char *caller_name = NULL;
+  char *locale_name;
   const char *program = basename (argv[0]);
+#ifdef USE_LDAP
+  char *binddn = NULL;
+#endif
 
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
@@ -238,7 +256,7 @@ main (int argc, char *argv[])
     use_service = "nis";
   else if (strcasecmp (program, "chsh") != 0)
     {
-      fprintf (stderr, _("%s: don't know what I should do.\n"), program);
+      fprintf (stderr, _("%s: Don't know what I should do.\n"), program);
       return 1;
     }
 
@@ -248,6 +266,10 @@ main (int argc, char *argv[])
       int option_index = 0;
       static struct option long_options[] =
 	{
+#ifdef USE_LDAP
+	  {"binddn",      required_argument, NULL, 'D' },
+#endif
+	  {"path",        required_argument, NULL, 'P' },
 	  {"service",     required_argument, NULL, 'r' },
 	  {"shell",       required_argument, NULL, 's' },
 	  {"list-shells", no_argument,       NULL, 'l' },
@@ -258,11 +280,13 @@ main (int argc, char *argv[])
 	  {NULL,          0,                 NULL, '\0'}
 	};
 
-      c = getopt_long (argc, argv, "s:r:lvuq", long_options, &option_index);
+      c = getopt_long (argc, argv, "D:P:s:r:lvuq",
+		       long_options, &option_index);
       if (c == (-1))
         break;
       switch (c)
         {
+
         case 'l':
 	  l_flag = 1;
 	  break;
@@ -289,11 +313,13 @@ main (int argc, char *argv[])
 	    use_service = "nisplus";
 	  else if (strcasecmp (optarg, "files") == 0)
 	    use_service = "files";
+#ifdef USE_LDAP
 	  else if (strcasecmp (optarg, "ldap") == 0)
 	    use_service = "ldap";
+#endif
 	  else
 	    {
-	      fprintf (stderr, _("Service \"%s\" not supported.\n"), optarg);
+	      fprintf (stderr, _("Service `%s' not supported.\n"), optarg);
 	      print_usage (stderr, program);
 	      return 1;
 	    }
@@ -301,11 +327,26 @@ main (int argc, char *argv[])
 	case 'q':
 	  silent = 1;
 	  break;
+#ifdef USE_LDAP
+        case 'D':
+          binddn = optarg;
+          break;
+#endif
+        case 'P':
+          if (uid != 0)
+            {
+              fprintf (stderr,
+                       _("Only root is allowed to specify another path\n"));
+              return E_NOPERM;
+            }
+          else
+            files_etc_dir = strdup (optarg);
+          break;
         case '\255':
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -366,7 +407,7 @@ main (int argc, char *argv[])
 	 data, too.  */
       if (argc == 1)
 	{
-	  char *user = argv[0];
+	  char *user = locale_to_utf8 (argv[0]);
 
 	  while (getpwnam_r (user, &resultbuf, buffer, buflen, &pw) != 0
 		 && errno == ERANGE)
@@ -375,9 +416,12 @@ main (int argc, char *argv[])
 	      buflen += 256;
 	      buffer = alloca (buflen);
 	    }
+
+	  free (user);
 	  if (!pw)
 	    {
-	      fprintf (stderr, _("%s: Unknown user `%s'.\n"), program, user);
+	      fprintf (stderr, _("%s: Unknown user `%s'.\n"),
+		       program, argv[0]);
 	      return 1;
 	    }
 	}
@@ -387,15 +431,17 @@ main (int argc, char *argv[])
 	{
 	  if (use_service)
 	    fprintf (stderr, _("%s: User `%s' is not known to service `%s'.\n"),
-		     program, pw->pw_name, use_service);
+		     program, utf8_to_locale (pw->pw_name), use_service);
 	  else
 	    fprintf (stderr, _("%s: Unknown user `%s'.\n"), program,
-		     pw->pw_name);
+		     utf8_to_locale (pw->pw_name));
 	  return 1;
 	}
+
+      locale_name = utf8_to_locale (pw_data->pw.pw_name);
     }
 
-  #ifdef WITH_SELINUX
+#ifdef WITH_SELINUX
   if (is_selinux_enabled () > 0)
     {
       if ((uid == 0) &&
@@ -407,7 +453,7 @@ main (int argc, char *argv[])
               (security_context_t) strdup (_("Unknown user context"));
 	  fprintf (stderr,
 		  _("%s: %s is not authorized to change the shell of `%s'.\n"),
-		   program, user_context, pw_data->pw.pw_name);
+		   program, user_context, locale_name);
 	  if (security_getenforce() > 0)
 	    {
 	      syslog (LOG_ALERT,
@@ -428,12 +474,17 @@ main (int argc, char *argv[])
 
 
   /* Only root is allowed to change shell for local users. */
-  if (uid && uid != pw_data->pw.pw_uid && pw_data->service == S_LOCAL)
+  if (uid && uid != pw_data->pw.pw_uid &&
+      (pw_data->service == S_LOCAL
+#ifdef USE_LDAP
+       || (pw_data->service == S_LDAP && binddn == NULL)
+#endif
+       ))
     {
       syslog (LOG_ERR, "%u cannot change shell for \"%s\"", uid,
 	      pw_data->pw.pw_name);
-      fprintf(stderr, _("You cannot change the shell for %s.\n"),
-	      pw_data->pw.pw_name);
+      fprintf (stderr, _("You cannot change the shell for %s.\n"),
+	       caller_name);
       free_user_t (pw_data);
       return 1;
     }
@@ -441,7 +492,7 @@ main (int argc, char *argv[])
   /* Normal user with restricted shell is not allowed to change it. */
   if (uid && restricted_shell (pw_data->pw.pw_shell))
     {
-      syslog (LOG_ERR, "User \"%s\" tries to change a restricted shell",
+      syslog (LOG_ERR, "User `%s' tries to change a restricted shell",
 	      pw_data->pw.pw_name);
       fprintf(stderr, _("You cannot change a restricted shell.\n"));
       free_user_t (pw_data);
@@ -449,12 +500,39 @@ main (int argc, char *argv[])
     }
 
   if (!silent)
-    printf (_("Changing login shell for %s.\n"), pw_data->pw.pw_name);
+    printf (_("Changing login shell for %s.\n"), locale_name);
 
   if (getlogindefs_bool ("CHFN_AUTH", 1) || pw_data->service != S_LOCAL)
     {
-      if (do_authentication (program, caller_name, pw_data) != 0)
-	return 1;
+#ifdef USE_LDAP
+      if (binddn && pw_data->service == S_LDAP)
+        {
+          /* A user tries to change data stored in a LDAP database and
+             knows the Manager dn, now we need the password from him.  */
+          ldap_session_t *session = create_ldap_session (LDAP_PATH_CONF);
+          char *cp;
+
+          if (session == NULL)
+            return E_FAILURE;
+
+          cp = getpass (_("Enter LDAP Password:"));
+
+          pw_data->binddn = strdup (binddn);
+
+          if (open_ldap_session (session) != 0)
+            return E_FAILURE;
+
+          if (ldap_authentication (session, NULL, binddn, cp) != 0)
+            return E_NOPERM;
+
+          close_ldap_session (session);
+
+          pw_data->oldclearpwd = strdup (cp);
+        }
+      else
+#endif /* USE_LDAP */
+	if (do_authentication (program, caller_name, pw_data) != 0)
+	  return 1;
       if (pw_data->service != S_LOCAL && get_old_clear_password (pw_data) != 0)
 	return 1;
     }
