@@ -352,18 +352,14 @@ main (int argc, char **argv)
 #endif
           break;
 	case 'g':
-          {
-            char *cp;
-
-            new_gid = strtoul (optarg, &cp, 10);
-            if (*cp != '\0')    /* invalid number */
-              {
-                fprintf (stderr, _("%s: Invalid numeric argument `%s'.\n"),
-                         program, optarg);
-                return E_BAD_ARG;
-              }
-            know_gid = 1;
-          }
+	  if (strtoid (optarg, &new_gid) == -1)    /* invalid number */
+	    {
+	      fprintf (stderr,
+		       _("%s: Invalid numeric argument `%s' for Group ID.\n"),
+		       program, optarg);
+	      return E_BAD_ARG;
+	    }
+	  know_gid = 1;
 	  break;
 	case 'o':
 	  non_unique = 1;
