@@ -44,6 +44,7 @@
 #include "public.h"
 #include "logindefs.h"
 #include "error_codes.h"
+#include "utf8conv.h"
 
 #ifndef _
 #define _(String) gettext (String)
@@ -145,17 +146,17 @@ main (int argc, char *argv[])
   if (argc == 1) /* Change primary group to new one.  */
     {
       struct group *grp;
-      long int gid;
+      gid_t gid;
       gid_t egid;
-      char *eptr;
       int is_member = 0;
       int ngroups, i;
       gid_t *grouplist;
+      char *utf8_arg = locale_to_utf8 (argv[0]);
 
       /* Try it as a group name, then a group id. */
-      if ((grp = getgrnam (argv[0])) == NULL &&
-	  ((gid = strtol (argv[0], &eptr, 10)) <= 0 ||
-	   *eptr != '\0' || (grp = getgrgid (gid)) == NULL))
+      if ((grp = getgrnam (utf8_arg)) == NULL &&
+	  (strtoid (utf8_arg, &gid) == -1 ||
+	   (grp = getgrgid (gid)) == NULL))
 	{
 	  fprintf (stderr, _("%s: bad group `%s'.\n"), program, argv[0]);
 	  return E_USAGE;
@@ -199,6 +200,8 @@ main (int argc, char *argv[])
       ngroups = getgroups (0, 0);
       if (ngroups == -1)
 	ngroups = NGROUPS_MAX;
+      if( (ngroups+1) <= 0)
+        return E_FAILURE;
       grouplist = malloc ((ngroups + 1) * sizeof (gid_t *));
       if (grouplist == NULL)
 	{
