@@ -174,10 +174,10 @@ convert_grpopt_to_name (const char *arg, char **grouplist, gid_t *groupid,
 
   if (isdigit (*arg))
     {
-      char *cp;
-      gid_t gid = strtoul (arg, &cp, 10);
+      gid_t gid;
+      int err = strtoid (arg, &gid);
 
-      if (*cp != '\0')		/* invalid number */
+      if (err == -1)		/* invalid number */
 	{
 	  fprintf (stderr, _("%s: Invalid numeric argument `%s'.\n"),
 		   program, arg);
@@ -281,10 +281,10 @@ load_defaults (const char *configfile, struct default_t *data)
       if (strncasecmp (cp, "GROUP=", 6) == 0)
 	{
 	  struct group *grp;
-	  char *ep;
-	  gid_t grpid = (gid_t) strtoul (&cp[6], &ep, 10);
+	  gid_t grpid;
 
-	  if (cp[6] != '\0' && *ep == '\0')	/* GID in numeric form */
+	  if (cp[6] != '\0' && strtoid (&cp[6], &grpid) == 0)
+	    /* GID in numeric form */
 	    data->group = grpid;
 	  else if ((grp = getgrnam (&cp[6])) != NULL)
 	    data->group = grp->gr_gid;
@@ -320,7 +320,9 @@ load_defaults (const char *configfile, struct default_t *data)
 	  char *ep;
 
 	  inactive = strtol (&cp[9], &ep, 10);
-	  if (*ep != '\0')	/* invalid number */
+	  if (*ep != '\0' ||
+	      ((inactive == LONG_MAX || inactive == LONG_MIN)
+	       && errno == ERANGE))	/* invalid number */
 	    {
 	      ++errors;
 	      fprintf (stderr, _("%s: Invalid numeric argument `%s' in %s.\n"),
@@ -1171,18 +1173,13 @@ main (int argc, char **argv)
 	  new_shell = optarg;
 	  break;
 	case 'u':
-	  {
-	    char *cp;
-
-	    new_uid = strtoul (optarg, &cp, 10);
-	    if (*cp != '\0')	/* invalid number */
-	      {
-		fprintf (stderr, _("%s: Invalid numeric argument `%s'.\n"),
-			 program, optarg);
-		return E_BAD_ARG;
-	      }
-	    know_uid = 1;
-	  }
+	  if (strtoid (optarg, &new_uid) == -1)	/* invalid number */
+	    {
+	      fprintf (stderr, _("%s: Invalid numeric argument `%s' for User ID.\n"),
+		       program, optarg);
+	      return E_BAD_ARG;
+	    }
+	  know_uid = 1;
 	  break;
 	case '\251':
 	case '\252':
