@@ -47,6 +47,7 @@
 
 #include "public.h"
 #include "group.h"
+#include "utf8conv.h"
 #include "logindefs.h"
 #include "read-files.h"
 #include "error_codes.h"
@@ -142,10 +143,9 @@ convert_grpopt_to_name (const char *arg, char **grouplist, gid_t *groupid,
 
   if (isdigit (*arg))
     {
-      char *cp;
-      gid_t gid = strtoul (arg, &cp, 10);
+      gid_t gid;
 
-      if (*cp != '\0')		/* invalid number */
+      if (strtoid (arg, &gid) == -1)	/* invalid number */
 	{
 	  fprintf (stderr, _("%s: invalid numeric argument `%s'.\n"),
 		   program, arg);
@@ -157,7 +157,8 @@ convert_grpopt_to_name (const char *arg, char **grouplist, gid_t *groupid,
 	{
 	  if (use_service)
 	    {
-	      fprintf (stderr, _("%s: group `%u' not found in service %s.\n"),
+	      fprintf (stderr,
+		       _("%s: group `%u' not found in service `%s'.\n"),
 		       program, gid, use_service);
 	      return E_NOTFOUND;
 	    }
@@ -175,13 +176,15 @@ convert_grpopt_to_name (const char *arg, char **grouplist, gid_t *groupid,
 	{
 	  if (use_service)
 	    {
-	      fprintf (stderr, _("%s: group `%s' not found in service %s.\n"),
-		       program, arg, use_service);
+	      fprintf (stderr,
+		       _("%s: group `%s' not found in service `%s'.\n"),
+		       program, utf8_to_locale (arg), use_service);
 	      return E_NOTFOUND;
 	    }
 	  else
 	    {
-	      fprintf (stderr, _("%s: unknown group `%s'.\n"), program, arg);
+	      fprintf (stderr, _("%s: unknown group `%s'.\n"), program,
+		       utf8_to_locale (arg));
 	      return E_BAD_ARG;
 	    }
 	}
@@ -393,7 +396,7 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
         {
           fprintf (stderr,
                    _("%s: ERROR: cannot find group `%s' anymore!.\n"),
-                   program, item->value);
+                   program, utf8_to_locale (item->value));
           if (retval == E_SUCCESS)
             retval = E_NOTFOUND;
         }
@@ -408,7 +411,7 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
                 {
                   fprintf (stderr,
                            _("%s: User not renamed in LDAP group `%s'.\n"),
-                           program, gr_data->gr.gr_name);
+                           program, utf8_to_locale (gr_data->gr.gr_name));
                   item = item->next;
                   free_group_t (gr_data);
                   retval = E_GRP_UPDATE;
@@ -421,7 +424,7 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
                 {
 		  fprintf (stderr,
 			   _("%s: User not renamed from LDAP group `%s'.\n"),
-			   program, gr_data->gr.gr_name);
+			   program, utf8_to_locale (gr_data->gr.gr_name));
 		  item = item->next;
 		  free_group_t (gr_data);
 		  retval = E_GRP_UPDATE;
@@ -439,7 +442,7 @@ rename_in_secondary_groups (user_t *pw_data, int have_extrapath)
             {
               fprintf (stderr,
                        _("%s: User not renamed in group `%s'.\n"),
-                       program, gr_data->gr.gr_name);
+                       program, utf8_to_locale (gr_data->gr.gr_name));
               retval = E_GRP_UPDATE;
             }
 
@@ -674,7 +677,7 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
         {
           fprintf (stderr,
                    _("%s: ERROR: cannot find group `%s' anymore!.\n"),
-                   program, item->value);
+                   program, utf8_to_locale (item->value));
           if (retval == E_SUCCESS)
             retval = E_NOTFOUND;
         }
@@ -699,7 +702,7 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
                 {
                   fprintf (stderr,
                            _("%s: User not removed from LDAP group `%s'.\n"),
-                           program, gr_data->gr.gr_name);
+                           program, utf8_to_locale (gr_data->gr.gr_name));
                   item = item->next;
                   free_group_t (gr_data);
                   retval = E_GRP_UPDATE;
@@ -718,7 +721,7 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
                     {
                       fprintf (stderr,
                                _("%s: User not removed from LDAP group `%s'.\n"),
-                               program, gr_data->gr.gr_name);
+                               program, utf8_to_locale (gr_data->gr.gr_name));
                       item = item->next;
                       free_group_t (gr_data);
                       retval = E_GRP_UPDATE;
@@ -736,7 +739,7 @@ remove_from_secondary_groups (user_t *pw_data, int have_extrapath,
             {
               fprintf (stderr,
                        _("%s: User not removed from group `%s'.\n"),
-                       program, gr_data->gr.gr_name);
+                       program, utf8_to_locale (gr_data->gr.gr_name));
               retval = E_GRP_UPDATE;
             }
 
@@ -761,7 +764,7 @@ main (int argc, char **argv)
   char *modify_account;
   char *new_group = NULL;
   gid_t new_gid = 0;
-  char *new_comment = "";
+  char *new_comment = NULL;
   char *new_home = NULL;
   char *old_home = NULL;
   char *new_shell = NULL;
@@ -810,7 +813,7 @@ main (int argc, char **argv)
 		       program, optarg);
 	      return E_BAD_ARG;
 	    }
-	  new_comment = optarg;
+	  new_comment = locale_to_utf8 (optarg);
 	  break;
 	case 'D':
 #ifdef USE_LDAP
@@ -824,7 +827,7 @@ main (int argc, char **argv)
 		       program, optarg);
 	      return E_BAD_ARG;
 	    }
-	  new_home = optarg;
+	  new_home = locale_to_utf8 (optarg);
 	  break;
 	case 'e':
 	  if (strcmp (optarg, "1969-12-31") == 0)
@@ -871,13 +874,13 @@ main (int argc, char **argv)
 	case 'G':
 	  /* Only save the arguments for later checking. We can find a
 	     -P <path> option later.  */
-	  new_groups_arg = optarg;
+	  new_groups_arg = locale_to_utf8 (optarg);
 	  break;
 	case 'g':
-	  new_group = optarg;
+	  new_group = locale_to_utf8 (optarg);
 	  break;
 	case 'l':
-	  new_login = optarg;
+	  new_login = locale_to_utf8 (optarg);
 	  break;
 	case 'L':
 	  if (unlock_password)
@@ -960,7 +963,7 @@ main (int argc, char **argv)
 #endif
 	  else
 	    {
-	      fprintf (stderr, _("Service \"%s\" not supported\n"), optarg);
+	      fprintf (stderr, _("Service `%s' not supported\n"), optarg);
 	      print_usage (stderr, program);
 	      return E_BAD_ARG;
 	    }
@@ -1027,13 +1030,13 @@ main (int argc, char **argv)
 	return E_NOPERM;
     }
 
-  modify_account = argv[0];
+  modify_account = locale_to_utf8 (argv[0]);
   /* Check, if the account we should modify exist.  */
   pw_data = do_getpwnam (modify_account, use_service);
   if (pw_data == NULL || pw_data->service == S_NONE)
     {				/* User does not exist.  */
       fprintf (stderr, _("%s: user `%s' does not exist.\n"),
-	       program, modify_account);
+	       program, utf8_to_locale (modify_account));
       return E_NOTFOUND;
     }
 
@@ -1083,7 +1086,8 @@ main (int argc, char **argv)
 		      (gid == pw_data->pw.pw_gid))
 		    {
 		      fprintf (stderr, _("%s: `%s' is primary group name.\n"),
-			       program, new_groups[new_groupscnt]);
+			       program,
+			       utf8_to_locale (new_groups[new_groupscnt]));
 		      /* return E_BAD_ARG; */
 		    }
 		  else
@@ -1113,7 +1117,7 @@ main (int argc, char **argv)
       is_logged_in (modify_account))
     {
       fprintf (stderr, _("%s: account `%s' is currently in use.\n"),
-	       program, modify_account);
+	       program, utf8_to_locale (modify_account));
       return E_USER_BUSY;
     }
 
@@ -1136,7 +1140,7 @@ main (int argc, char **argv)
       if (check_name (new_login) != 0)
 	{
 	  fprintf (stderr, _("%s: Invalid account name `%s'.\n"),
-		   program, new_login);
+		   program, utf8_to_locale (new_login));
 	  return E_BAD_ARG;
 	}
       else
@@ -1145,7 +1149,7 @@ main (int argc, char **argv)
 	      (have_extrapath && files_getpwnam (new_login) != NULL))
 	    {
 	      fprintf (stderr, _("%s: Account `%s' already exists.\n"),
-		       program, new_login);
+		       program, utf8_to_locale (new_login));
 	      return E_NAME_IN_USE;
 	    }
 	}
@@ -1187,16 +1191,26 @@ main (int argc, char **argv)
       if (new_shell)
 	pw_data->new_shell = strdup (new_shell);
 
-      if (pw_data->use_shadow && (know_inactive || know_expire))
+      if (know_inactive || know_expire)
 	{
-	  pw_data->spn = pw_data->sp;
-	  pw_data->sp_changed = 1;
+	  if (pw_data->use_shadow)
+	    {
+	      pw_data->spn = pw_data->sp;
+	      pw_data->sp_changed = 1;
 
-	  if (know_inactive)
-	    pw_data->spn.sp_inact = new_inactive;
+	      if (know_inactive)
+		pw_data->spn.sp_inact = new_inactive;
 
-	  if (know_expire)
-	    pw_data->sp.sp_expire = new_expire;
+	      if (know_expire)
+		pw_data->spn.sp_expire = new_expire;
+	    }
+	  else
+	    {
+	      fprintf (stderr,
+		       _("%s: Shadow passwords required for -e and -f\n"),
+		       program);
+	      return E_NO_SHADOW;
+	    }
 	}
 
 #ifdef USE_LDAP
@@ -1280,12 +1294,12 @@ main (int argc, char **argv)
 	      if (use_service == NULL)
 		fprintf (stderr,
 			 _("%s: ERROR: cannot find group `%s' anymore!.\n"),
-			 program, new_groups[i]);
+			 program, utf8_to_locale (new_groups[i]));
 	      else
 		fprintf (stderr,
 			 _
 			 ("%s: Cannot find group `%s' in service `%s', ignored.\n"),
-			 program, new_groups[i], use_service);
+			 program, utf8_to_locale (new_groups[i]), use_service);
 	      retval = E_NOTFOUND;
 	    }
 	  else
@@ -1310,7 +1324,8 @@ main (int argc, char **argv)
 		    {
 		      fprintf (stderr,
 			       _("%s: User not added to LDAP group `%s'.\n"),
-			       program, gr_data->gr.gr_name);
+			       program,
+			       utf8_to_locale (gr_data->gr.gr_name));
 		      free_group_t (gr_data);
 		      retval = E_GRP_UPDATE;
 		      continue;
@@ -1328,7 +1343,8 @@ main (int argc, char **argv)
 			{
 			  fprintf (stderr,
 				   _("%s: User not added to LDAP group `%s'.\n"),
-				   program, gr_data->gr.gr_name);
+				   program,
+				   utf8_to_locale (gr_data->gr.gr_name));
 			  free_group_t (gr_data);
 			  retval = E_GRP_UPDATE;
 			  continue;
@@ -1345,7 +1361,8 @@ main (int argc, char **argv)
 		{
 		  fprintf (stderr,
 			   _("%s: User not added to LDAP group `%s'.\n"),
-			   program, gr_data->gr.gr_name);
+			   program,
+			   utf8_to_locale (gr_data->gr.gr_name));
 		  retval = E_GRP_UPDATE;
 		}
 	      /* Make sure, written group changes will be active now.  */
@@ -1369,15 +1386,20 @@ main (int argc, char **argv)
     {
       int fd;
       int ret;
+      char *home = new_home ? new_home : old_home;
 
-      /* If know_uid is set, change UID on filesystem of file
-	 in the home directory.  */
-      ret = chown_dir_rec (new_home ? new_home : old_home,
-			   pw_data->pw.pw_uid, new_uid,
-			   pw_data->pw.pw_gid,
-			   new_group ? new_gid : pw_data->pw.pw_gid);
-      if (ret != 0)
-	retval = E_FAILURE;
+      /* Only change the UID of the home directory, if it exist.
+	 Else ignore it.  */
+      if (access (home, F_OK) == 0)
+	{
+	  /* If know_uid is set, change UID on filesystem of file
+	     in the home directory.  */
+	  ret = chown_dir_rec (home, pw_data->pw.pw_uid, new_uid,
+			       pw_data->pw.pw_gid,
+			       new_group ? new_gid : pw_data->pw.pw_gid);
+	  if (ret != 0)
+	    retval = E_FAILURE;
+	}
 
       /* Relocate the "lastlog/faillog" entries for the user. */
       if ((fd = open ("/var/log/faillog", O_RDWR)) >= 0)
