@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 Thorsten Kukuk
+/* Copyright (C) 2003, 2004 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -169,7 +169,7 @@ edit (const char *file, const char *program, int shadow)
 
   if (lock_database () != 0)
     {
-      fprintf (stderr, _("Cannot lock %s: already locked.\n"), file);
+      fprintf (stderr, _("Cannot lock `%s': already locked.\n"), file);
       return E_PWDBUSY;
     }
 
@@ -187,7 +187,7 @@ edit (const char *file, const char *program, int shadow)
 #endif
   if (new_fd == -1)
     {
-      fprintf (stderr, _("Can't create %s: %m\n"), tmpname);
+      fprintf (stderr, _("Can't create `%s': %m\n"), tmpname);
       ulckpwdf ();
       free (tmpname);
       return E_FAILURE;
@@ -247,7 +247,7 @@ edit (const char *file, const char *program, int shadow)
 	}
       if (fstat (old_fd, &orig))
 	{
-	  fprintf (stderr, _("Can't stat %s: %m\n"), file);
+	  fprintf (stderr, _("Can't stat `%s': %m\n"), file);
 	  return E_FAILURE;
 	}
       close (old_fd);
@@ -256,7 +256,7 @@ edit (const char *file, const char *program, int shadow)
 
   if (stat (tmpname, &before))
     {
-      fprintf (stderr, _("Can't stat %s: %m\n"), tmpname);
+      fprintf (stderr, _("Can't stat `%s': %m\n"), tmpname);
       return E_FAILURE;
     }
 
@@ -270,7 +270,7 @@ edit (const char *file, const char *program, int shadow)
 
   if (stat (tmpname, &after))
     {
-      fprintf (stderr, _("Can't stat %s: %m\n"), tmpname);
+      fprintf (stderr, _("Can't stat `%s': %m\n"), tmpname);
       return E_FAILURE;
     }
 
@@ -376,7 +376,7 @@ main (int argc, char *argv[])
           return 0;
 	case 'V': /* RH compatibility.  */
         case 'v':
-          print_version (program, "2003");
+          print_version (program, "2004");
           return 0;
         case 'u':
           print_usage (stdout, program);
